package pro.litvinovg.libreoffice.metadata;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;

import org.json.JSONException;
import org.json.JSONObject;

import com.sun.star.beans.PropertyVetoException;
import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.text.XText;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextField;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.UnoRuntime;
import static pro.litvinovg.libreoffice.metadata.Document.*;
public class OutlineElement {

	private String elementName;
	private ArrayList<MetadataElement> metadata = null;
	private XTextRange textRange;
	
	public String getName() {
		return elementName;
	}
	public boolean isEmpty() {
		if (metadata.isEmpty()) {
			return true;
		} 
		for (int i = 0; i < metadata.size(); i++) {
			if (!metadata.get(i).getName().isEmpty() && !metadata.get(i).getValue().isEmpty()) {
				return false;
			}
		}
		return true;
	}

	public OutlineElement(XTextRange textRange, String name) {
		this.textRange = textRange;
		this.elementName = name;
		this.metadata = new ArrayList<MetadataElement>();
	}
	
	public Object[][] metadataToArray() {
		Object [] [] result = null; 
		int size = metadata.size();
		if (metadata.size() == 0) {
			result = new Object[][] {
				{"", ""},
				{"", ""},
				{"", ""},
			};
		} else {
			result = new Object [size] [2];
			for (int i = 0; i < size; i++) {
				result[i][0] = metadata.get(i).getName();
				result[i][1] = metadata.get(i).getValue();
			}
		}
		return result;
	}
	
	public void readMetadata(String data) {
		try {
  		JSONObject json = new JSONObject(data);
  		 Set<String> nums = json.keySet();
  			for (String number : nums) {
  				JSONObject meta = (JSONObject) json.get(number);
  				Set<String> names = meta.keySet();
  				for (String name: names) {
  					String value = meta.getString(name);
  					metadata.add(new MetadataElement(name, value));
  				}	
  			}
		} catch (JSONException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
private String metadataToString() {
    JSONObject json = new JSONObject();
    Integer i = 0;
    for (MetadataElement element : metadata) {
    	String name = element.getName();
    	String value = element.getValue();
    	if (!name.isEmpty() && !value.isEmpty()) {
        JSONObject meta = new JSONObject();
    		meta.put(element.getName(), element.getValue());
    		json.put(i.toString(), meta);
    		i++;
    	}
    }
		return json.toString();
	}

public void writeMetadata(Object annotation) {
	XPropertySet properties = UnoRuntime.queryInterface(XPropertySet.class, annotation);
	XText text = textRange.getText();
	try {
		properties.setPropertyValue("Author", METADATA_EXTENSION);
		properties.setPropertyValue("Content", metadataToString());

		XTextRange annotationRange = null;
		annotationRange = textRange;
		XTextField textField = UnoRuntime.queryInterface(XTextField.class, annotation);
		text.insertTextContent( annotationRange, textField, false );
	} catch (IllegalArgumentException e) {
		e.printStackTrace();
	} catch (UnknownPropertyException e) {
		e.printStackTrace();
	} catch (PropertyVetoException e) {
		e.printStackTrace();
	} catch (WrappedTargetException e) {
		e.printStackTrace();
	}
}
public void setMetadata(ArrayList<MetadataElement> metadataInput) {
	this.metadata.clear();
	this.metadata.addAll(metadataInput);
}
	


}
